from requests_toolbelt.threaded import pool

from collectionsdownload.iiifhandler import IIIFCollection, IIIFPresentation, IIIFOCRResource, IIIFImageResource


class TestIIIFCollection():

    def test_items(self):
        requests = pool.Pool.from_urls(['https://iiif.onb.ac.at/presentation/collection/Karlstadt'])
        requests.join_all()
        collections = [
            request.json()
            for request in requests.responses()
        ]

        collection = IIIFCollection(collections[0])

        ids = [id_ for id_ in collection.presentation_ids]
        assert all([id_.__class__ is str for id_ in ids])
        assert all([id_.startswith('http') for id_ in ids])
        n_ids = len(ids)
        assert 1 < n_ids < 5

        collection = IIIFCollection(collections[0], limit_presentations=2)
        assert len(list(collection.presentation_ids)) == 2

        collection = IIIFCollection(collections[0], skip_first_n_presentations=1)
        assert len(list(collection.presentation_ids)) == n_ids - 1

        collection = IIIFCollection(collections[0], limit_presentations=3, skip_first_n_presentations=1)
        assert len(list(collection.presentation_ids)) == 3


class TestIIIFResource():

    def setup_class(self):
        """
        Threading for on erequest, because, I do not know, we will do it like this later!
        :return:
        """
        self.presentation_id = 'https://iiif.onb.ac.at/presentation/ABO/Z171322203/manifest/'
        request_pool = pool.Pool.from_urls([self.presentation_id])
        request_pool.join_all()
        response_jsons = [response.json() for response in request_pool.responses()]
        self.presentation_data = response_jsons[0]


    def test_get_meta_data_by_label(self):
        presentation = IIIFPresentation(self.presentation_id, self.presentation_data)
        barcode = presentation.get_meta_data_by_label('Barcode')
        assert barcode == presentation.barcode
        assert barcode.__class__ is str
        assert barcode.startswith('Z')
        assert barcode == 'Z171322203'

        assert presentation.get_meta_data_by_label('XORG') is None

    def test_ocr_resources(self):

        presentation = IIIFPresentation(self.presentation_id, self.presentation_data)
        ocrs = list(presentation.ocr_resources)
        assert all([ocr.__class__ is IIIFOCRResource for ocr in ocrs])
        assert all([ocr.resource_id.startswith('http') for ocr in ocrs])

        ocr_n = len(ocrs)

        presentation = IIIFPresentation(self.presentation_id, self.presentation_data, limit_ocrs=2)
        assert len(list(presentation.ocr_resources)) == 2

        presentation = IIIFPresentation(self.presentation_id, self.presentation_data, skip_first_n_ocrs=1)
        assert len(list(presentation.ocr_resources)) == ocr_n - 1

        presentation = IIIFPresentation(self.presentation_id, self.presentation_data, skip_first_n_ocrs=1, limit_ocrs=2)
        assert len(list(presentation.ocr_resources)) == 2

    def test_image_resources(self):

        presentation = IIIFPresentation(self.presentation_id, self.presentation_data)
        images = list(presentation.image_resources)
        assert all([image.__class__ is IIIFImageResource for image in images])
        assert all([image.resource_id.startswith('http') for image in images])
        assert images[0].resource_id == 'https://iiif.onb.ac.at/images/ABO/Z171322203/00000001/full/full/0/native.jpg'
        assert images[-1].resource_id == 'https://iiif.onb.ac.at/images/ABO/Z171322203/00000284/full/full/0/native.jpg'
        image_n = len(images)

        presentation = IIIFPresentation(self.presentation_id, self.presentation_data, limit_images=2)
        assert len(list(presentation.image_resources)) == 2

        presentation = IIIFPresentation(self.presentation_id, self.presentation_data, skip_first_n_images=1)
        assert len(list(presentation.image_resources)) == image_n - 1

        presentation = IIIFPresentation(self.presentation_id, self.presentation_data, skip_first_n_images=1, limit_images=2)
        assert len(list(presentation.image_resources)) == 2
